#!/usr/bin/env python3
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()
import platform
import sys
import os

import click

from .__version__ import __version__
from .util.consoleeffects import Colors
from .util.data_cache import load_role_cache

try:
    import lxml.etree as ET
except ImportError:
    if platform.system() == "Windows":
        print(
            "awslogin will not run on your machine yet.  Please follow the instructions at https://github.com/byu-oit/awslogin/releases/tag/lxml to get it running."
        )
        sys.exit(1)
    else:
        raise


from .util.data_cache import get_status, load_cached_adfs_auth, remove_cached_adfs_auth
from .login import cached_login, non_cached_login


# Enable VT Mode on windows terminal code from:
# https://bugs.python.org/issue29059
# This works not sure if it the best way or not
if platform.system().lower() == "windows":
    from ctypes import windll, c_int, byref

    stdout_handle = windll.kernel32.GetStdHandle(c_int(-11))
    mode = c_int(0)
    windll.kernel32.GetConsoleMode(c_int(stdout_handle), byref(mode))
    mode = c_int(mode.value | 4)
    windll.kernel32.SetConsoleMode(c_int(stdout_handle), mode)

completion = load_role_cache()


def account_completion(ctx, args, incomplete):
    try:
        return [k for k in completion.keys() if incomplete in k]
    except:  # noqa: E722
        return []


def role_completion(ctx, args, incomplete):
    try:
        return [k for k in completion[args[1]] if incomplete in k]
    except IndexError:
        return []


@click.command()
@click.version_option(version=__version__)
@click.option(
    "-a", "--account", help="Account to login with", autocompletion=account_completion
)
@click.option(
    "-r", "--role", help="Role to use after login", autocompletion=role_completion
)
@click.option(
    "-p",
    "--profile",
    default="default",
    help="Profile to use store credentials. Defaults to default",
)
@click.option(
    "--region", default="us-west-2", help="The AWS region you will be hitting"
)
@click.option(
    "-s",
    "--status",
    is_flag=True,
    default=False,
    help="Display current logged in status. Use profile all to see all statuses",
)
@click.option(
    "--logout",
    is_flag=True,
    default=False,
    help="Logout of ADFS cached session only. Does not log out of any active profiles.",
)
@click.option("--proxy", default="", help="Specify http/https proxy to use for login")
def cli(account, role, profile, region, status, logout, proxy):
    # Display status and exit if the user specified the "-s" flag
    if status:
        get_status(profile)
        return

    if logout:
        remove_cached_adfs_auth()
        print("{}Terminated ADFS Session{}".format(Colors.yellow, Colors.normal))
        return

    if proxy:
        os.environ["http_proxy"] = proxy
        os.environ["HTTP_PROXY"] = proxy
        os.environ["https_proxy"] = proxy
        os.environ["HTTPS_PROXY"] = proxy

    # Use cached SAML assertion if already logged in, else login to ADFS
    adfs_auth_result = load_cached_adfs_auth()
    if adfs_auth_result:
        cached_login(account, role, profile, region, adfs_auth_result)
        pass
    else:
        non_cached_login(account, role, profile, region)


if __name__ == "__main__":
    cli()



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import open
from future import standard_library

standard_library.install_aliases()
import base64
import configparser
import datetime
import os
import json
import pickle
from os.path import expanduser

from .consoleeffects import Colors


def load_cached_adfs_auth():
    file = _aws_file("credentials")
    config = _open_config_file(file)
    section = "all"
    if config.has_section(section) and config.has_option(section, "adfs_auth"):
        unpickled = pickle.loads(
            base64.urlsafe_b64decode(config[section]["adfs_auth"].encode())
        )
        return unpickled
    else:
        return None


def remove_cached_adfs_auth():
    file = _aws_file("credentials")
    config = _open_config_file(file)
    section = "all"
    if config.has_section(section) and config.has_option(section, "adfs_auth"):
        config.remove_option(section, "adfs_auth")
        with open(file, "w") as configfile:
            config.write(configfile)


def cache_adfs_auth(adfs_auth_result):
    _create_aws_dir_if_not_exists()
    file = _aws_file("credentials")
    config = _open_config_file(file)
    pickled = base64.urlsafe_b64encode(pickle.dumps(adfs_auth_result)).decode()
    config["all"] = {"adfs_auth": pickled}
    with open(file, "w") as configfile:
        config.write(configfile)


def get_status(profile="default"):
    file = _aws_file("config")
    config = _open_config_file(file)
    if profile == "all":
        for x in config:
            if x == "DEFAULT":
                continue
            message = _get_status_message(config, x)
            print("{}{} - {}".format(Colors.white, x, message))
        return
    else:
        if profile != "default":
            profile = "profile {}".format(profile)
        if config.has_section(profile):
            message = _get_status_message(config, profile)
            print(message)
        else:
            print(
                "{}Couldn't find profile: {}{}".format(
                    Colors.red, profile, Colors.normal
                )
            )
        return


def load_last_netid(profile):
    file = _aws_file("config")
    config = _open_config_file(file)
    if config.has_section(profile) and config.has_option(profile, "adfs_netid"):
        return config[profile]["adfs_netid"]
    else:
        return ""


def write_to_config_file(profile, net_id, region, role, account, sessionDuration):
    _create_aws_dir_if_not_exists()
    if profile != "default":
        profile = "profile {}".format(profile)
    file = _aws_file("config")
    session_duration_delta = datetime.timedelta(seconds=sessionDuration)
    expires = datetime.datetime.now() + session_duration_delta
    config = _open_config_file(file)
    if not net_id and config.get(profile, "adfs_netid", fallback=None):
        net_id = config[profile]["adfs_netid"]
    config[profile] = {
        "region": region,
        "adfs_role": "{}@{}".format(role, account),
        "adfs_expires": expires.strftime("%m-%d-%Y %H:%M"),
    }
    if net_id:
        config[profile]["adfs_netid"] = net_id
    with open(file, "w") as configfile:
        config.write(configfile)


def write_to_cred_file(profile, aws_session_token):
    _create_aws_dir_if_not_exists()
    file = _aws_file("credentials")
    config = _open_config_file(file)
    config[profile] = {
        "aws_access_key_id": aws_session_token["Credentials"]["AccessKeyId"],
        "aws_secret_access_key": aws_session_token["Credentials"]["SecretAccessKey"],
        "aws_session_token": aws_session_token["Credentials"]["SessionToken"],
    }
    with open(file, "w") as configfile:
        config.write(configfile)


def write_role_cache(roles):
    _create_aws_dir_if_not_exists()
    output = {}
    for account in roles:
        output[account] = [k for k in roles[account]]
    file = _aws_file(".awslogin_accountrole_cache.json")
    with open(file, "w") as f:
        json.dump(output, f, indent=2)


def load_role_cache():
    file = _aws_file(".awslogin_accountrole_cache.json")
    try:
        with open(file, "r") as f:
            return json.load(f)
    except:  # noqa: E722
        return {}


def _aws_file(file_name):
    return "{}/.aws/{}".format(expanduser("~"), file_name)


def _create_aws_dir_if_not_exists(directory="{}/.aws".format(expanduser("~"))):
    if not os.path.exists(directory):
        os.makedirs(directory)


def _open_config_file(file):
    config = configparser.ConfigParser()
    config.read(file)
    return config


def _get_status_message(config, profile):
    if config.has_option(profile, "adfs_role") and config.has_option(
        profile, "adfs_expires"
    ):
        expires = _check_expired(config[profile]["adfs_expires"])
        account_name = "{}{}".format(Colors.cyan, config[profile]["adfs_role"])
        if expires == "Expired":
            expires_msg = "{}{} at: {}".format(
                Colors.red, expires, config[profile]["adfs_expires"]
            )
        else:
            expires_msg = "{}{} at: {}".format(
                Colors.yellow, expires, config[profile]["adfs_expires"]
            )
        return "{} {}- {}{}".format(
            account_name, Colors.white, expires_msg, Colors.normal
        )
    else:
        return "{}Couldn't find status info{}".format(Colors.red, Colors.normal)


def _check_expired(expires):
    expires = datetime.datetime.strptime(expires, "%m-%d-%Y %H:%M")
    if expires > datetime.datetime.now():
        return "Expires"
    else:
        return "Expired"



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import input
from future import standard_library

standard_library.install_aliases()
import getpass

from .auth.adfs_auth import authenticate
from .auth.assume_role import ask_which_role_to_assume
from .auth.assume_role import assume_role
from .auth.saml import get_account_names, get_saml_assertion
from .util.consoleeffects import Colors
from .util.data_cache import (
    load_last_netid,
    write_to_config_file,
    write_to_cred_file,
    cache_adfs_auth,
)

STS_SESSION_TIMEOUT = 32400  # Valid for 9 hours


def cached_login(account, role, profile, region, adfs_auth_result):
    saml_assertion = get_saml_assertion(adfs_auth_result)

    if not saml_assertion:  # Not logged in anymore, so need to re-login
        non_cached_login(account, role, profile, region)
    else:  # Still logged in and got a SAML assertion properly
        _perform_login(
            adfs_auth_result, saml_assertion, account, role, profile, None, region
        )


def non_cached_login(account, role, profile, region):
    # Get the federated credentials from the user
    net_id, username = _get_user_ids(profile)
    password = getpass.getpass()
    print("")

    ####
    # Authenticate against ADFS with DUO MFA
    ####
    adfs_auth_result = authenticate(username, password)

    # Overwrite and delete the credential variables, just for safety
    username = "##############################################"
    password = "##############################################"
    del username
    del password

    saml_assertion = get_saml_assertion(adfs_auth_result)
    _perform_login(
        adfs_auth_result, saml_assertion, account, role, profile, net_id, region
    )


def _perform_login(
    adfs_auth_result, saml_assertion, account, role, profile, net_id, region
):
    account_roles_to_assume = _prompt_for_roles_to_assume(saml_assertion, account, role)

    # Assume all requested roles and set in the environment
    for account_role_to_assume in account_roles_to_assume:
        aws_session_token = assume_role(
            account_role_to_assume, saml_assertion.assertion, STS_SESSION_TIMEOUT
        )

        # If assuming roles across all accounts, then use the account name as the profile name
        if account == "all":
            profile = account_role_to_assume.account_name

        write_to_cred_file(profile, aws_session_token)
        write_to_config_file(
            profile,
            net_id,
            region,
            account_role_to_assume.role_name,
            account_role_to_assume.account_name,
            STS_SESSION_TIMEOUT,
        )
        _print_status_message(account_role_to_assume)

    # By caching the ADFS auth information we can re-assume roles without reauthenticating until the ADFS session expires
    cache_adfs_auth(adfs_auth_result)


def _prompt_for_roles_to_assume(saml_assertion, account, role):
    account_names = get_account_names(saml_assertion)
    account_roles_to_assume = ask_which_role_to_assume(
        account_names, saml_assertion.get_principal_roles(), account, role
    )
    return account_roles_to_assume


def _print_status_message(assumed_role):
    if assumed_role.role_name == "Admin":
        print(
            "Now logged into {}{}{}@{}{}{}".format(
                Colors.red,
                assumed_role.role_name,
                Colors.white,
                Colors.yellow,
                assumed_role.account_name,
                Colors.normal,
            )
        )
    else:
        print(
            "Now logged into {}{}{}@{}{}{}".format(
                Colors.cyan,
                assumed_role.role_name,
                Colors.white,
                Colors.yellow,
                assumed_role.account_name,
                Colors.normal,
            )
        )


def _get_user_ids(profile):
    # Ask for NetID, or use cached if user doesn't specify another
    cached_netid = load_last_netid(profile)
    if cached_netid:
        net_id_prompt = "BYU Net ID [{}{}{}]: ".format(
            Colors.blue, cached_netid, Colors.normal
        )
    else:
        net_id_prompt = "BYU Net ID: "
    net_id = input(net_id_prompt) or cached_netid

    # Append the ADFS-required "@byu.local" to the Net ID
    if "@byu.local" in net_id:
        print("{}@byu.local{} is not required".format(Colors.lblue, Colors.normal))
        username = net_id
    else:
        username = "{}@byu.local".format(net_id)

    return net_id, username



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['byu_awslogin', 'byu_awslogin.auth', 'byu_awslogin.util']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.7,<5.0',
 'boto3>=1.9,<2.0',
 'click>=7.0,<8.0',
 'configparser>=3.7,<5.0',
 'future>=0.17.1,<0.19.0',
 'lxml>=4.3,<5.0',
 'prompt-toolkit>=2.0,<3.0',
 'requests>=2.21,<3.0']

entry_points = \
{'console_scripts': ['awslogin = byu_awslogin.index:cli']}

setup_kwargs = {
    'name': 'byu-awslogin',
    'version': '0.15.2',
    'description': "An aws-adfs spinoff that fits BYU's needs",
    'long_description': "# AWSLOGIN\nPython script for CLI and SDK access to AWS via ADFS while requiring MFA\naccess using <https://duo.com/>\n\n# History and Purpose\n\nBYU used to use the great [aws-adfs](https://github.com/venth/aws-adfs)\nCLI tool to login to our AWS accounts. It worked great, especially the\nDUO 2FA support. Eventually, we decided to write our own similar tool\nbut make it BYU-specific so that we could tailor it to our needs (which\nbasically means hard-code certain BYU-specific things) and remove some\nof the required parameters. Since this tool will be used by BYU\nemployees only we had that option. We then morphed it a little more for\nour use cases. This isn't something that you could use outside of BYU,\nsorry.\n\n# DUO 2FA Requirements\nIn order for Duo 2FA to work properly Automatic Push needs to be enabled.\n\n# Installation\n\n  - Python 3.6+ is recommended as python2 is EOL January 2020.\n  - It is highly recommended to use an application like [Pipx](https://pipxproject.github.io/pipx/) to install and use python cli applications.\n  - Follow the pipx [installation documentation](https://pipxproject.github.io/pipx/installation/) then simply run `pipx install byu_awslogin`\n  - Experimental Binaries are available on the releases page. These are new and in testing [Releases](https://gihtub.com/byu-oit/awslogin/releases)\n  - See the [installation options](https://github.com/byu-oit/awslogin/blob/master/INSTALLATION_OPTIONS.md) For additional options\n    page for step by step instructions for installing in various environments\n\n# Upgrading\n\nIf you already have byu\\_awslogin install and are looking to upgrade\nsimply run\n\n`pip3 install --upgrade byu_awslogin` or `pip install --upgrade\nbyu_awslogin` as appropriate for your python installation\n\n# Usage\n\nawslogin defaults to the default profile in your \\~/.aws/config and\n\\~/.aws/credentials files. **\\*If you already have a default profile you\nwant to save in your \\~/.aws files make sure to do that before running\nawslogin.**\\*\n\nOnce you're logged in, you can execute commands using the AWS CLI or AWS\nSDK. Try running `aws s3 ls`.\n\nCurrently, AWS temporary credentials are only valid for 1 hour. We cache\nyour ADFS session, however, so you can just re-run `awslogin` again to\nget a new set of AWS credentials without logging in again to ADFS. Your\nADFS login session is valid for 8 hours, after which time you'll be\nrequired to login to ADFS again to obtain a new session.\n\nTo switch accounts after you've already authenticated to an account,\njust run awslogin again and select a new account/role combination.\n\nTo use it:\n\n  - Run `awslogin` and it will prompt you for the AWS account and role\n    to use.\n  - Run `awslogin --account <account name> --role <role name>` to skip\n    the prompting for account and name. You could specify just one of\n    the arguments as well.\n  - Run `awslogin --profile <profile name>` to specifiy an alternative\n    profile\n  - Run `awslogin --region <region name>` to specify a different region.\n    The default region is *us-west-2*.\n  - Run `awslogin --status` for the current status of the default\n    profile\n  - Run `awslogin --status --profile dev` for the current status of the\n    dev profile\n  - Run `awslogin --status --profile all` for the current status of the\n    all profiles\n  - Run `awslogin --logout` to logout of a cached ADFS session\n  - Run `awslogin --version` to display the running version of awslogin\n  - Run `awslogin --help` for full help message\n\n# Bash or ZSH Completion\nBash:\n- Run the following: `_AWSLOGIN_COMPLETE=source awslogin > ~/_awslogin` Then add `source /path/to/_awslogin` to .bashrc\n\nZSH:\n- Run the following: `_AWSLOGIN_COMPLETE=source_zsh awslogin > ~/_awslogin` Then add `source /path/to/_awslogin` to .zshrc\n\nAlternatively put the `_awslogin` in your `/etc/bash_completion.d` or similiar directory (`~/.zfunc`) to load at shells startup\n\nTo test if it works run awslogin at least once for the account and role cache to populate. On next login `awslogin -a [TAB][TAB]` should output available accounts and `awslogin -a {some account} -r [TAB][TAB]` should output available roles for the selected account\n\nLimitation: Accounts and Role completion at the CLI is loaded from a cache file. This file will be updated anytime awslogin is ran.\n\n# Reporting bugs or requesting features\n\n  - Enter an issue on the github repo.\n  - Or, even better if you can, fix the issue and make a pull request.\n\n# Deploying changes\n\n  - Update the version in `pyproject.toml` and `__version__.py`\n  - Commit the changes and push.\n  - Build binaries\n  - Create a new release (add binaries and sha256sums.txt) with the version number and Github Actions will build, test and publish\n\n# TODO\n\n  - Write tests\n    - Write more tests to increase overall coverage\n",
    'author': 'BYU OIT Application Development',
    'author_email': 'it@byu.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/byu-oit/awslogin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)
